

/* ***********************************************************************
                Naval Air Warfare Center / Aircraft Division              
                        Patuxent Naval Air Station                         
 ========================================================================

   File:           capability.c                                         
                                                                        
   Classification: UNCLASSIFIED                                   
                                                                        
   Description:    This file contains the functions, methods, and global
      variables needed to extract and manipulate sensors defined in the TDB
			
   Written by:     Alex Harper
*********************************************************************** */


#include "parse_tdb.h"


/* ***********************************************************
   This is the contructor to the capability_type class
*********************************************************** */
capability_type::capability_type() {

   typeflag = 0;
   antenna_table = transmission_loss_table = NULL;
   flags = 0;
   receiver_freq = 1.0;
   duty_cycle = 1.0;
   pulse_ratio = 0.0;
   one_m2_detect_range = 0.0;
   pixel_fov = 0.0;
   prf = 0.0;
   pulse_width = 1.0;

   receiver_noise = receiver_loss = receiver_threshold = operating_loss = 0;
   transmitter_loss = 0;
   sensor_type = SENSOR_OTHER;
   minmax_el[0] = -HALFPI;
   minmax_el[1] = HALFPI;
   minmax_az[0] = -PI;
   minmax_az[1] = PI;

   presented_area_freq = 5.17e14; // default to mid-range for optical
   noise_equiv_irradiance_flag = 0;
}


/* ***********************************************************
   This is the destructor to the capability_type class
*********************************************************** */
capability_type::~capability_type() {
   
   if (antenna_table)
      delete antenna_table;

   if (transmission_loss_table)
      delete transmission_loss_table;
}


/* ***********************************************************
  This method searches for a specific capability

  char *x // name of capability to search for
*********************************************************** */
capability_type *capability_type::find(char *x) {

   if (!name.stringcmp(x))
      return this;
      
   if (next)
      return ((capability_type *)next)->find(x);

   return NULL;
}


/* ***********************************************************
  This method extracts data from the capability block

   FILE *infile,    // input file
   char *token,     // last processed token
   int *errorflag   // error flag
*********************************************************** */
void capability_type::extract_tdbdat(FILE *infile, char *token, int *errorflag) {

   double db, hz, m, m2;
   range_type *rtr;
   
   *errorflag = get_sweg_token(infile, token);

   while (*errorflag) {

      if (!strcmp(token, TOKEN_CAPABILITY)) {
         *errorflag = get_sweg_token(infile, token);
         return;
      }
      
      if (!strcmp(token, TOKEN_ANTENNA_PATTERN)) {
      
         if (antenna_table)
            delete antenna_table;
    
         *errorflag = get_sweg_token(infile, token);		// "DIMENSION"
         *errorflag = get_sweg_token(infile, token);		// #
         *errorflag = get_sweg_token(infile, token);		// type

         if (!strcmp(token, TOKEN_FREQ))
            antenna_table = new frequency_type;
         else if (!strcmp(token, TOKEN_AZ))
            antenna_table = new azimuth_type;
         else
            antenna_table = new elevation_type;

         antenna_table->extract_tdbdat(infile, token, errorflag);
         *errorflag = get_sweg_token(infile, token);		// antenna-pattern
         *errorflag = get_sweg_token(infile, token);
         continue;
      }

      if (!strcmp(token, TOKEN_DUTY_CYCLE)) {
         *errorflag = get_sweg_token(infile, token);
         db = atof(token);

         *errorflag = get_sweg_token(infile, token);  // uom - ignore

         duty_cycle = db;

         *errorflag = get_sweg_token(infile, token);
         continue;
      }

      if (!strcmp(token, TOKEN_PULSE_COMPRESSION_RATIO)) {
         *errorflag = get_sweg_token(infile, token);
         db = atof(token);

         *errorflag = get_sweg_token(infile, token);  // uom - DB, (DB)

         pulse_ratio = db;

         *errorflag = get_sweg_token(infile, token);
         continue;
      }

      if (!strcmp(token, TOKEN_ONE_M2_DETECT_RNG)) {
         *errorflag = get_sweg_token(infile, token);
         one_m2_detect_range = atof(token);

         *errorflag = get_sweg_token(infile, token);
         one_m2_detect_range *= get_scale(token, TOKEN_PMP);
         flags |= FLAG_ONE_M2;

         *errorflag = get_sweg_token(infile, token);
         continue;
      }

/*
      if (!strcmp(token, TOKEN_RCVR_BANDWIDTH)) {
         *errorflag = get_sweg_token(infile, token);
         receiver_freq = atof(token);
         
         *errorflag = get_sweg_token(infile, token);
         receiver_freq *= get_scale(token, TOKEN_PHZP);

         *errorflag = get_sweg_token(infile, token);
         continue;
      }
*/

      if (!strcmp(token, TOKEN_SNR_RCVR_FREQS)) {
         *errorflag = get_sweg_token(infile, token);

         while (*errorflag) {

            if (!strcmp(token, TOKEN_SNR_RCVR_FREQS))
               break;

            if (!strcmp(token, TOKEN_SEARCH) ||
                !strcmp(token, TOKEN_TRACKING) ||
                !strcmp(token, TOKEN_GUIDANCE)) {
    
               *errorflag = get_sweg_token(infile, token);	// value
               hz = atof(token);
               *errorflag = get_sweg_token(infile, token);	// units
               hz *= get_scale(token, TOKEN_PHZP);

               if (!(flags & FLAG_ACQUISITION))
                  receiver_freq = hz;
       
               *errorflag = get_sweg_token(infile, token);
               continue; 
            }
    
            if (!strcmp(token, TOKEN_ACQUISITION)) {
    
               *errorflag = get_sweg_token(infile, token);	// value
               hz = atof(token);
               *errorflag = get_sweg_token(infile, token);	// units
               hz *= get_scale(token, TOKEN_PHZP);

               flags |= FLAG_ACQUISITION;
               receiver_freq = hz;
       
               *errorflag = get_sweg_token(infile, token);
               continue; 
            }
    
            *errorflag = get_sweg_token(infile, token);
         }
 
         *errorflag = get_sweg_token(infile, token);
         continue; 
      }

      if (!strcmp(token, TOKEN_DETECTION_SENSITIVITIES)) {
         *errorflag = get_sweg_token(infile, token);

         while (*errorflag) {

            if (!strcmp(token, TOKEN_DETECTION_SENSITIVITIES))
               break;

            if (!strcmp(token, TOKEN_SENSING_THRESHOLD)) {
    
               *errorflag = get_sweg_token(infile, token);	// value
               db = atof(token);
               *errorflag = get_sweg_token(infile, token);	// units

               if (strcmp(token, TOKEN_PDBP) && strcmp(token, TOKEN_DB))
                  db = 10.0*log10(db);

               flags |= FLAG_RECEIVER_THRESHOLD;
               receiver_threshold = db;
       
               if (!(flags & FLAG_RECEIVER_NOISE))
                  receiver_noise = db;
  
               if (!(flags & FLAG_RECEIVER_LOSSES))
                  receiver_loss = db;
  
               *errorflag = get_sweg_token(infile, token);
               continue; 
            }
    
            if (!strcmp(token, TOKEN_RECEIVER_NOISE)) {
               *errorflag = get_sweg_token(infile, token);	// value
               db = atof(token);
               *errorflag = get_sweg_token(infile, token);	// units
//               db *= get_scale(token, TOKEN_PDBP);

               flags |= FLAG_RECEIVER_NOISE;
               receiver_noise = db;
       
               if (!(flags & FLAG_RECEIVER_THRESHOLD))
                  receiver_threshold = db;
  
               if (!(flags & FLAG_RECEIVER_LOSSES))
                  receiver_loss = db;
  
               *errorflag = get_sweg_token(infile, token);
               continue; 
            }
    
            if (!strcmp(token, TOKEN_INTERNAL_LOSSES)) {
               *errorflag = get_sweg_token(infile, token);	// value
               db = atof(token);
               *errorflag = get_sweg_token(infile, token);	// units
//               db *= get_scale(token, TOKEN_PDBP);

               flags |= FLAG_RECEIVER_LOSSES;
               receiver_loss = db;
       
               if (!(flags & FLAG_RECEIVER_THRESHOLD))
                  receiver_threshold = db;
  
               if (!(flags & FLAG_RECEIVER_NOISE))
                  receiver_noise = db;
  
               *errorflag = get_sweg_token(infile, token);
               continue; 
            }
    
            if (!strcmp(token, TOKEN_OPERATING_LOSSES)) {
               *errorflag = get_sweg_token(infile, token);	// value
               db = atof(token);
               *errorflag = get_sweg_token(infile, token);	// units
//               db *= get_scale(token, TOKEN_PDBP);

               operating_loss = db;

               *errorflag = get_sweg_token(infile, token);
               continue;	 
            }
	    
            if (!strcmp(token, TOKEN_NOISE_EQUIV_IRRADIANCE)) {
               *errorflag = get_sweg_token(infile, token);	// value
               db = atof(token);
               *errorflag = get_sweg_token(infile, token);	// units
//               db *= get_scale(token, TOKEN_PWPERM2P);

               noise_equiv_irradiance = db;
               noise_equiv_irradiance_flag = 1;

               *errorflag = get_sweg_token(infile, token);
               continue;	 
            }
	    
            if (!strcmp(token, TOKEN_POST_LOCKON_THRESHOLD)) {
               *errorflag = get_sweg_token(infile, token);	// value
               db = atof(token);
               *errorflag = get_sweg_token(infile, token);	// units
//               db *= get_scale(token, TOKEN_PDBP);

               if (!(flags & FLAG_RECEIVER_LOSSES))
                  receiver_loss = db;
  
               if (!(flags & FLAG_RECEIVER_THRESHOLD))
                  receiver_threshold = db;
  
               if (!(flags & FLAG_RECEIVER_NOISE))
                  receiver_noise = db;

               *errorflag = get_sweg_token(infile, token);	// next
               continue; 
            }

            *errorflag = get_sweg_token(infile, token);
         }
 
         *errorflag = get_sweg_token(infile, token);
         continue; 
      }

      if (!strcmp(token, TOKEN_INTERNAL_LOSS)) {
         *errorflag = get_sweg_token(infile, token);	// value
         transmitter_loss = atof(token);
       
         *errorflag = get_sweg_token(infile, token);	// units
//         transmitter_loss *= get_scale(token, TOKEN_PDBP);
    
         *errorflag = get_sweg_token(infile, token);
         continue; 
      }

      if (!strcmp(token, TOKEN_PEAK_POWER_OUTPUT)) {
         *errorflag = get_sweg_token(infile, token);	// value
         transmitter_power = atof(token);
       
         *errorflag = get_sweg_token(infile, token);	// units
         transmitter_power *= get_scale(token, TOKEN_PWATTSP);
    
         *errorflag = get_sweg_token(infile, token);
         continue; 
      }
               
      if (!strcmp(token, TOKEN_PULSE_REPETITION_FREQ)) {
         *errorflag = get_sweg_token(infile, token);	// value
         prf = atof(token);
       
         *errorflag = get_sweg_token(infile, token);	// units
         prf *= get_scale(token, TOKEN_PHZP);
    
         *errorflag = get_sweg_token(infile, token);
         continue; 
      }
               
      if (!strcmp(token, TOKEN_PULSE_WIDTH)) {
         *errorflag = get_sweg_token(infile, token);	// value
         pulse_width = atof(token);
       
         *errorflag = get_sweg_token(infile, token);	// units
         pulse_width *= get_scale(token, TOKEN_PWATTSP);
    
         *errorflag = get_sweg_token(infile, token);
         continue; 
      }
               
      if (!strcmp(token, TOKEN_SNR_ANGULAR_LIMITS)) {

         *errorflag = get_sweg_token(infile, token);

         while (*errorflag) {

            if (!strcmp(token, TOKEN_SNR_ANGULAR_LIMITS))
               break;

            if (!strcmp(token, TOKEN_AZ_LIMIT)) {
	    
               *errorflag = get_sweg_token(infile, token);
               minmax_az[0] = atof(token);

               *errorflag = get_sweg_token(infile, token);
               minmax_az[0] *= get_scale(token, TOKEN_PRADIANSP);
	            minmax_az[1] = -minmax_az[0];

               *errorflag = get_sweg_token(infile, token);
               continue;	 
            }
	    
            if (!strcmp(token, TOKEN_EL_LIMIT)) {
	    
               *errorflag = get_sweg_token(infile, token);
               minmax_el[0] = atof(token);

               *errorflag = get_sweg_token(infile, token);
               minmax_el[0] *= get_scale(token, TOKEN_PRADIANSP);
	            minmax_el[1] = -minmax_el[0];

               *errorflag = get_sweg_token(infile, token);
               continue;	 
            }
	    
            if (!strcmp(token, TOKEN_LOWER_EL_LIMIT)) {
	    
               *errorflag = get_sweg_token(infile, token);
               minmax_el[0] = atof(token);

               *errorflag = get_sweg_token(infile, token);
               minmax_el[0] *= get_scale(token, TOKEN_PRADIANSP);
	            
               *errorflag = get_sweg_token(infile, token);
               continue;	 
            }
	    
            if (!strcmp(token, TOKEN_UPPER_EL_LIMIT)) {
	    
               *errorflag = get_sweg_token(infile, token);
               minmax_el[1] = atof(token);

               *errorflag = get_sweg_token(infile, token);
               minmax_el[1] *= get_scale(token, TOKEN_PRADIANSP);
	            
               *errorflag = get_sweg_token(infile, token);
               continue;	 
            }
	    
            if (!strcmp(token, TOKEN_ACQUISITION)) {
	    
               *errorflag = get_sweg_token(infile, token);	// value
               hz = atof(token);
               *errorflag = get_sweg_token(infile, token);	// units
               hz *= get_scale(token, TOKEN_PHZP);

               flags |= FLAG_ACQUISITION;
               receiver_freq = hz;
	       
               *errorflag = get_sweg_token(infile, token);
               continue;	 
            }
	    
            *errorflag = get_sweg_token(infile, token);
         }
	 
         *errorflag = get_sweg_token(infile, token);
         continue;	 
      }

      if (!strcmp(token, TOKEN_SNR_CHARACTERISTICS)) {

         *errorflag = get_sweg_token(infile, token);

         while (*errorflag) {

            if (!strcmp(token, TOKEN_SNR_CHARACTERISTICS))
               break;

            if (!strcmp(token, TOKEN_ACOUSTIC)) {
    
               sensor_type = SENSOR_SONAR;
  
               *errorflag = get_sweg_token(infile, token);
               continue; 
            }
    
            if (!strcmp(token, TOKEN_ELECTRO_MAGNETIC)) {
	    
               sensor_type = SENSOR_EM;
		  
               *errorflag = get_sweg_token(infile, token);
               continue;	 
            }
	    
            if (!strcmp(token, TOKEN_INFRA_RED)) {
	    
               sensor_type = SENSOR_IR;
		  
               *errorflag = get_sweg_token(infile, token);
               continue;	 
            }
	    
            *errorflag = get_sweg_token(infile, token);
         }
 
         *errorflag = get_sweg_token(infile, token);
         continue; 
      }
               
      if (!strcmp(token, TOKEN_RNG_ALT_CAPABILITY)) {
         get_sweg_token(infile, token);
         get_sweg_token(infile, token);
         get_sweg_token(infile, token);

         get_sweg_token(infile, token);  // unit
         m = get_scale(token, TOKEN_M);

         *errorflag = get_sweg_token(infile, token);

         while (*errorflag) {

            if (!strcmp(token, TOKEN_RNG_ALT_CAPABILITY) || !strcmp(token, TOKEN_MIN_ALT))
               break;
       
            range_list.append(rtr = new range_type, NULL);
            rtr->range = atof(token) * m;
            *errorflag = get_sweg_token(infile, token);
         }

         rtr = (range_type *)range_list.head;

         while (*errorflag) {

            if (!strcmp(token, TOKEN_END)) {
               *errorflag = get_sweg_token(infile, token);
               if (!strcmp(token, TOKEN_RNG_ALT_CAPABILITY))
                  break;
            }

            if (!strcmp(token, TOKEN_RNG_ALT_CAPABILITY))
               break;
	       
            *errorflag = get_sweg_token(infile, token);
            m = get_scale(token, TOKEN_M);

            *errorflag = get_sweg_token(infile, token);
            *errorflag = get_sweg_token(infile, token);
            m2 = get_scale(token, TOKEN_M);

            *errorflag = get_sweg_token(infile, token);
            rtr->minmax[0] = atof(token)*m;

            *errorflag = get_sweg_token(infile, token);
            rtr->minmax[0] = atof(token)*m2;

            rtr = (range_type *)rtr->next;
            *errorflag = get_sweg_token(infile, token);
         }

         *errorflag = get_sweg_token(infile, token);
         continue; 
      }

      if (!strcmp(token, TOKEN_TRANSMISSION_LOSS)) {
      
         if (transmission_loss_table)
            delete transmission_loss_table;
	    
         *errorflag = get_sweg_token(infile, token);		// "DIMENSION"
         *errorflag = get_sweg_token(infile, token);		// #
         *errorflag = get_sweg_token(infile, token);		// type

         antenna_table = new frequency_type;

         antenna_table->extract_tdbdat(infile, token, errorflag);
         *errorflag = get_sweg_token(infile, token);		// transmission-loss
         *errorflag = get_sweg_token(infile, token);
         continue;
      }

      if (!strcmp(token, TOKEN_PRESENTED_AREA_FREQ)) {
      	    
         *errorflag = get_sweg_token(infile, token);
         presented_area_freq = atof(token);

         *errorflag = get_sweg_token(infile, token);

         get_scale(token, TOKEN_PHZP);

         *errorflag = get_sweg_token(infile, token);
         continue;
      }

      if (!strcmp(token, TOKEN_PIXEL_FIELD_OF_VIEW)) {
      	    
         *errorflag = get_sweg_token(infile, token);
         pixel_fov = atof(token);

         *errorflag = get_sweg_token(infile, token);  // STERADIAN

         *errorflag = get_sweg_token(infile, token);
         continue;
      }

      *errorflag = get_sweg_token(infile, token);
   }

}
